(function () {
  /*jshint multistr: true */  // allows for multi-line strings
  "use strict";

  // Console log shows in Chrome console, can use to add breakpoint to debug
  //
  console.log('template plugin');

  var element = EMBED.getRootElement(),                   // Attach to the component unique id root element
      data    = EMBED.getComponent().schema.data;         // Initialize object to link data with manifest

  // function to create alpha channel background colors from properties on plugin
  var setColor = function(value, defaultColor)
  {
    var color;
    if (value && value.hasOwnProperty('hex'))
    {
      color = value;
    }
    else
    {
      color = value ? JSON.parse(value) : defaultColor;
    }
    var opacity = Math.round(parseFloat(color.rgb.a) * 255);
    opacity = opacity.toString(16);
    opacity = ('00' + opacity).slice(-2);

    return color.hex + opacity;
  };

  // find html elements within this container
  //
  var containerElem = element.find("#iContainer");

  // Color/Opacity constants Ex: white.hex
  const BLACK  = {hex:"#000000FF",rgb:{a:1}},
        GRAY   = {hex:"#9b9b9bFF",rgb:{a:1}},
        WHITE  = {hex:"#FFFFFFFF",rgb:{a:1}},
        RED    = {hex:"#FF0000FF",rgb:{a:1}},
        GREEN  = {hex:"#00FF00FF",rgb:{a:1}},
        BLUE   = {hex:"#0000FFFF",rgb:{a:1}},
        YELLOW = {hex:"#FFFF00FF",rgb:{a:1}},
        CLEAR  = {hex:"#FFFFFF00",rgb:{a:0}};


  // Connect to input and output elements for plugin
  //
  var inputValue = data.inputValue;                                   // data source input
  
  // Connect to other properties on the plugin
  //
  var styling = data.styling;                                         // styling
  var backgroundColor = setColor(styling.backgroundColor,GRAY);       // background color
  var fontColor = setColor(styling.fontColor,BLACK);                  // font color
  var fontSize = styling.fontSize;                                    // font size
  var fontFamily = styling.fontFamily;                                // font family

  // Function to re-apply styling if required
  //
  var applyStyling = function()
  {
    containerElem[0].style.color = fontColor;
    containerElem[0].style.backgroundColor = backgroundColor;
    containerElem[0].style.fontSize = fontSize.toString() + "pt";
    containerElem[0].style.fontFamily = fontFamily;
  };

  // Update value in plugin from data source
  //
  var updateWidget = function(value)
  {
    // Set the value in the HTML with the value provided
    //
    containerElem.text(value);
  };


  // Main body
  //
  applyStyling();


  // DATA HANDLER
  //
  // cases to handle data from queries, manual, globals or formula from OpsHub
  //
  var formulaFirstTime = true;
  if ((inputValue.type === 'formula') && formulaFirstTime)
  {
    updateWidget(inputValue.formula.data.value[0].value);
    formulaFirstTime = false;
  }

  // Data link
  if (inputValue.type === 'manual')
  {
    updateWidget(inputValue.manual);
  }
  else if (EMBED.fieldTypeIsGlobal(inputValue))
  {
    // Fetch data
    var value = EMBED.getGlobalData(inputValue);
    // update widget
    updateWidget(value);

    EMBED.subscribeFieldToGlobalChange(inputValue, function(data)
    {
      updateWidget(data);
    });
  }
  else if(EMBED.fieldTypeIsQuery(inputValue))
  {
    EMBED.subscribeFieldToQueryChange(inputValue, function(rows)
    {
      var output = rows[0][EMBED.getQueryOutput(inputValue)[0].fieldName];
      updateWidget(output);
    });
  }
  else if(EMBED.fieldTypeIsFormula(inputValue))
  {
    // Fetch data
    var value = EMBED.getFormulaData(inputValue);
    // update widget
    updateWidget(value);
    EMBED.subscribeFieldToFormulaChange(inputValue, function(data)
    {
      updateWidget(data);
    });
  }
})();
